<?php

namespace Islhd\Supplier\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Islhd\Supplier\Http\Requests\ContractRequest;
use Islhd\Supplier\Interfaces\ContractRepositoryInterface;
use Islhd\Supplier\Models\Contract;
use Islhd\Supplier\Forms\Contract as Form;

/**
 * APIController  class for contract.
 */
class ContractAPIController extends BaseController
{

    /**
     * Initialize contract resource controller.
     *
     * @param type ContractRepositoryInterface $contract
     *
     * @return null
     */
    public function __construct(ContractRepositoryInterface $contract)
    {
        parent::__construct();
        $this->repository = $contract;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Supplier\Repositories\Criteria\ContractResourceCriteria::class);
    }

    /**
     * Display a list of contract.
     *
     * @return Response
     */
    public function index(ContractRequest $request)
    {
        return $this->repository
            ->setPresenter(\Islhd\Supplier\Repositories\Presenter\ContractPresenter::class)
            ->paginate();
    }

    /**
     * Display contract.
     *
     * @param Request $request
     * @param Model   $contract
     *
     * @return Response
     */
    public function show(ContractRequest $request, Contract $contract)
    {
        return $contract->setPresenter(\Islhd\Supplier\Repositories\Presenter\ContractListPresenter::class);
        ;
    }

    /**
     * Create new contract.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ContractRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('supplier::contract.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('supplier/contract/' . $contract->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/contract');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the contract.
     *
     * @param Request $request
     * @param Model   $contract
     *
     * @return Response
     */
    public function update(ContractRequest $request, Contract $contract)
    {
        try {
            $data = $request->all();

            $contract->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('supplier::contract.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('supplier/contract/' . $contract->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/contract/' . $contract->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the contract.
     *
     * @param Model   $contract
     *
     * @return Response
     */
    public function destroy(ContractRequest $request, Contract $contract)
    {
        try {
            $contract->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('supplier::contract.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('supplier/contract/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/contract/' . $contract->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
