<?php

namespace Islhd\Supplier\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Islhd\Supplier\Http\Requests\ContractRequest;
use Islhd\Supplier\Interfaces\ContractRepositoryInterface;
use Islhd\Supplier\Models\Contract;

/**
 * Resource controller class for contract.
 */
class ContractResourceController extends BaseController
{

    /**
     * Initialize contract resource controller.
     *
     * @param type ContractRepositoryInterface $contract
     *
     * @return null
     */
    public function __construct(ContractRepositoryInterface $contract)
    {
        parent::__construct();
        $this->repository = $contract;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Supplier\Repositories\Criteria\ContractResourceCriteria::class);
    }

    /**
     * Display a list of contract.
     *
     * @return Response
     */
    public function index(ContractRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Islhd\Supplier\Repositories\Presenter\ContractPresenter::class)
                ->$function();
        }

        $contracts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('supplier::contract.names'))
            ->view('supplier::contract.index', true)
            ->data(compact('contracts', 'view'))
            ->output();
    }

    /**
     * Display contract.
     *
     * @param Request $request
     * @param Model   $contract
     *
     * @return Response
     */
    public function show(ContractRequest $request, Contract $contract)
    {

        if ($contract->exists) {
            $view = 'supplier::contract.show';
        } else {
            $view = 'supplier::contract.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('supplier::contract.name'))
            ->data(compact('contract'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new contract.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ContractRequest $request)
    {

        $contract = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('supplier::contract.name')) 
            ->view('supplier::contract.create', true) 
            ->data(compact('contract'))
            ->output();
    }

    /**
     * Create new contract.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ContractRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $contract                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('supplier::contract.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('supplier/contract/' . $contract->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/supplier/contract'))
                ->redirect();
        }

    }

    /**
     * Show contract for editing.
     *
     * @param Request $request
     * @param Model   $contract
     *
     * @return Response
     */
    public function edit(ContractRequest $request, Contract $contract)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('supplier::contract.name'))
            ->view('supplier::contract.edit', true)
            ->data(compact('contract'))
            ->output();
    }

    /**
     * Update the contract.
     *
     * @param Request $request
     * @param Model   $contract
     *
     * @return Response
     */
    public function update(ContractRequest $request, Contract $contract)
    {
        try {
            $attributes = $request->all();

            $contract->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('supplier::contract.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('supplier/contract/' . $contract->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('supplier/contract/' . $contract->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the contract.
     *
     * @param Model   $contract
     *
     * @return Response
     */
    public function destroy(ContractRequest $request, Contract $contract)
    {
        try {

            $contract->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('supplier::contract.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('supplier/contract/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('supplier/contract/' . $contract->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple contract.
     *
     * @param Model   $contract
     *
     * @return Response
     */
    public function delete(ContractRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('supplier::contract.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('supplier/contract'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/supplier/contract'))
                ->redirect();
        }

    }

    /**
     * Restore deleted contracts.
     *
     * @param Model   $contract
     *
     * @return Response
     */
    public function restore(ContractRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('supplier::contract.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/supplier/contract'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/supplier/contract/'))
                ->redirect();
        }

    }

}
