<?php

namespace Islhd\Supplier\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Islhd\Supplier\Http\Requests\SupplierRequest;
use Islhd\Supplier\Interfaces\SupplierRepositoryInterface;
use Islhd\Supplier\Models\Supplier;
use Islhd\Supplier\Forms\Supplier as Form;

/**
 * APIController  class for supplier.
 */
class SupplierAPIController extends BaseController
{

    /**
     * Initialize supplier resource controller.
     *
     * @param type SupplierRepositoryInterface $supplier
     *
     * @return null
     */
    public function __construct(SupplierRepositoryInterface $supplier)
    {
        parent::__construct();
        $this->repository = $supplier;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Islhd\Supplier\Repositories\Criteria\SupplierResourceCriteria::class);
    }

    /**
     * Display a list of supplier.
     *
     * @return Response
     */
    public function index(SupplierRequest $request)
    {
        return $this->repository
            ->setPresenter(\Islhd\Supplier\Repositories\Presenter\SupplierPresenter::class)
            ->paginate();
    }

    /**
     * Display supplier.
     *
     * @param Request $request
     * @param Model   $supplier
     *
     * @return Response
     */
    public function show(SupplierRequest $request, Supplier $supplier)
    {
        return $supplier->setPresenter(\Islhd\Supplier\Repositories\Presenter\SupplierListPresenter::class);
        ;
    }

    /**
     * Create new supplier.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(SupplierRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('supplier::supplier.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('supplier/supplier/' . $supplier->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/supplier');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the supplier.
     *
     * @param Request $request
     * @param Model   $supplier
     *
     * @return Response
     */
    public function update(SupplierRequest $request, Supplier $supplier)
    {
        try {
            $data = $request->all();

            $supplier->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('supplier::supplier.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('supplier/supplier/' . $supplier->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/supplier/' . $supplier->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the supplier.
     *
     * @param Model   $supplier
     *
     * @return Response
     */
    public function destroy(SupplierRequest $request, Supplier $supplier)
    {
        try {
            $supplier->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('supplier::supplier.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('supplier/supplier/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('supplier/supplier/' . $supplier->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
